
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE GET_BG_FG(JDATE, JTIME, F)

C-----------------------------------------------------------------------
C Function:
C   Get the function value and gradient of the background operator at the
C   initial time

C Revision History:
C   Apr 2007 Kumaresh Singh and Adrian Sandu at Virginia Tech: created
C   Jun 2009 Peter Percell at UH-IMAQS: rewrote
C-----------------------------------------------------------------------

      USE GRID_CONF             ! horizontal & vertical domain specifications
      USE FOURDVAR_MOD          ! 4DVAR data

      IMPLICIT NONE

C Parameters:
      REAL(8), PARAMETER :: BG_WGT = 50.0D0

C Arguments:
      INTEGER, INTENT(IN) :: JDATE ! current model date, coded YYYYDDD
      INTEGER, INTENT(IN) :: JTIME ! current model time, coded HHMMSS
      REAL(8), INTENT(INOUT) :: F ! cumulative value of cost function (for local domain)

C Local Variables:
      LOGICAL, SAVE :: FIRSTIME = .TRUE.

      CHARACTER(16) :: PNAME = 'GET_BG_FG'
      CHARACTER(96) :: XMSG = ' '

      LOGICAL, SAVE :: S_BND_PE, E_BND_PE, N_BND_PE, W_BND_PE

      INTEGER, SAVE :: SZ       ! local zero index for southern boundary
      INTEGER, SAVE :: EZ       ! local zero index for eastern boundary
      INTEGER, SAVE :: NZ       ! local zero index for northern boundary
      INTEGER, SAVE :: WZ       ! local zero index for western boundary

      INTEGER :: C, R, L, S     ! column, row, layer and species indices

      REAL(8) :: D

C-----------------------------------------------------------------------

      IF ( FIRSTIME ) THEN
         FIRSTIME = .FALSE.

         S_BND_PE = ( ROWSX_PE(1, MYPE+1) == 1 )
         E_BND_PE = ( COLSX_PE(2, MYPE+1) == GL_NCOLS )
         N_BND_PE = ( ROWSX_PE(2, MYPE+1) == GL_NROWS )
         W_BND_PE = ( COLSX_PE(1, MYPE+1) == 1 )

         SZ = 0
         EZ = NCOLS + 1
         NZ = NCOLS + NROWS + 3
         WZ = 2 * NCOLS + NROWS + 4
      END IF

C Update the cost function F and gradient LGRID_BC_SF
      DO S = 1, N_CNTL_SPC
         DO L = 1, NLAYS

C South boundary
            IF ( S_BND_PE ) THEN
               DO C = 1, NCOLS
                  D = (BC_SF(SZ+C, L, S) - 1.0) / BG_WGT
                  F = F + D**2 / 2.0D0
                  LGRID_BC_SF(SZ+C, L, S) =
     &                 LGRID_BC_SF(SZ+C, L, S) + D / BG_WGT
               END DO
            END IF

C East boundary
            IF ( E_BND_PE ) THEN
               DO R = 1, NROWS
                  D = (BC_SF(EZ+R, L, S) - 1.0) / BG_WGT
                  F = F + D**2 / 2.0D0
                  LGRID_BC_SF(EZ+R, L, S) =
     &                 LGRID_BC_SF(EZ+R, L, S) + D / BG_WGT
               END DO
            END IF

C North boundary
            IF ( N_BND_PE ) THEN
               DO C = 1, NCOLS
                  D = (BC_SF(NZ+C, L, S) - 1.0) / BG_WGT
                  F = F + D**2 / 2.0D0
                  LGRID_BC_SF(NZ+C, L, S) =
     &                 LGRID_BC_SF(NZ+C, L, S) + D / BG_WGT
               END DO
            END IF

C West boundary
            IF ( W_BND_PE ) THEN
               DO R = 1, NROWS
                  D = (BC_SF(WZ+R, L, S) - 1.0) / BG_WGT
                  F = F + D**2 / 2.0D0
                  LGRID_BC_SF(WZ+R, L, S) =
     &                 LGRID_BC_SF(WZ+R, L, S) + D / BG_WGT
               END DO
            END IF

         END DO
      END DO

      END SUBROUTINE GET_BG_FG
