
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header: /home/aqm/ppercell/aq/cmaq_4.5/models/CCTM/src/hadv/hppm/rdbcon.F,v 1.1.1.1 2005/09/09 18:56:06 sjr Exp $

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE RDBCON_IO_UNITS(JDATE, JTIME, TSTEP, LVL, BCON)

C-----------------------------------------------------------------------
C Function:
C   Read boundary concentrations data for advection in I/O units

C Revision History:
C   Jeff - Aug 1997 Based on beta version, keep in ppmV units (not aerosols)
C   Jeff - Dec 97 - add CMIN
C   Jeff - Apr 98 - fix conversion/coupling for aerosol number species
C   Jeff - Apr 01 - dyn alloc - Use PINTERB for boundary data - assume the met
C                   data could come from a larger file, but not the conc cata
C   23 Jun 03 J.Young: for layer dependent advection tstep
C   31 Jan 05 J.Young: dyn alloc - establish both horizontal & vertical
C                      domain specifications in one module
C-----------------------------------------------------------------------

      USE GRID_CONF             ! horizontal & vertical domain specifications
      USE FOURDVAR_MOD          ! 4DVAR data

      IMPLICIT NONE

C Includes:
      INCLUDE SUBST_GC_ADV      ! gas chem advection species and map table
      INCLUDE SUBST_AE_ADV      ! aerosol advection species and map table
      INCLUDE SUBST_NR_ADV      ! non-react advection species and map table
      INCLUDE SUBST_TR_ADV      ! tracer advection species and map table
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IOFDESC     ! file header data structure
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_FILES_ID    ! file name parameters

C Parameters:
      REAL, PARAMETER :: CMIN = 1.0E-30

C Arguments:
      INTEGER, INTENT(IN) :: JDATE ! current model date, coded YYYYDDD
      INTEGER, INTENT(IN) :: JTIME ! current model time, coded HHMMSS
      INTEGER, INTENT(IN) :: TSTEP ! timestep
      INTEGER, INTENT(IN) :: LVL ! layer
      REAL, INTENT(OUT)   :: BCON(:, :) ! boundary concentrations

C File Variables:
      REAL :: BBUF(1:NBNDY, 1:NLAYS) ! bcon file buffer

C External Functions not declared in IODECL3.EXT:
      INTEGER, EXTERNAL :: SEC2TIME, TIME2SEC
#ifdef parallel
      LOGICAL, EXTERNAL :: INTERPB
#endif

C Local Variables:
      CHARACTER(16) :: PNAME = 'RDBCON_IO_UNITS'
      CHARACTER(96) :: XMSG = ' '

      CHARACTER(16) :: BLNK = ' '

      INTEGER :: MDATE          ! mid-advection date
      INTEGER :: MTIME          ! mid-advection time

      INTEGER :: S              ! species index
      INTEGER :: STRT, FINI     ! loop limits

      CHARACTER(16) :: VNAME    ! variable name

C-----------------------------------------------------------------------

      MDATE  = JDATE
      MTIME  = JTIME
      CALL NEXTIME(MDATE, MTIME, SEC2TIME(TIME2SEC(TSTEP)/2))

      BCON(:, :) = 0.0

C Read & interpolate boundary concentrations
      STRT = 1
      FINI = N_GC_ADV
      DO S = STRT, FINI
         VNAME = BCNAME_ADV(S)
         IF ( VNAME .NE. BLNK ) THEN
            IF ( .NOT. INTERPB(BNDY_GASC_1, VNAME, PNAME,
     &           MDATE, MTIME, NBNDY*NLAYS, BBUF) ) THEN
               XMSG = 'Could not read ' // TRIM(VNAME) //
     &              ' from ' // BNDY_GASC_1
               CALL M3EXIT(PNAME, MDATE, MTIME, XMSG, XSTAT1)
            ELSE                ! found bc's (PPM) on file
               BCON(1:NBNDY, S) = BBUF(1:NBNDY, LVL)
            END IF
         ELSE
            BCON(1:NBNDY, S) = CMIN
         END IF
      END DO

      STRT = N_GC_ADV + 1
      FINI = N_GC_ADV + N_AE_ADV
      DO S = STRT, FINI
         VNAME = BCNAME_ADV(S)
         IF ( VNAME .NE. BLNK ) THEN
            IF ( .NOT. INTERPB(BNDY_AERO_1, VNAME, PNAME,
     &           MDATE, MTIME, NBNDY*NLAYS, BBUF) ) THEN
               XMSG = 'Could not read ' // TRIM(VNAME) //
     &              ' from ' // BNDY_AERO_1
               CALL M3EXIT(PNAME, MDATE, MTIME, XMSG, XSTAT1)
            ELSE                ! found bc's (microgram/m**3, m**2/m**3, or number/m**3)
               BCON(1:NBNDY, S) = BBUF(1:NBNDY, LVL)
            END IF
         ELSE
            BCON(1:NBNDY, S) = CMIN
         END IF
      END DO

      STRT = N_GC_ADV + N_AE_ADV + 1
      FINI = N_GC_ADV + N_AE_ADV + N_NR_ADV
      DO S = STRT, FINI
         VNAME = BCNAME_ADV(S)
         IF ( VNAME .NE. BLNK ) THEN
            IF ( .NOT. INTERPB(BNDY_NONR_1, VNAME, PNAME,
     &           MDATE, MTIME, NBNDY*NLAYS, BBUF) ) THEN
               XMSG = 'Could not read ' // TRIM(VNAME) //
     &              ' from ' // BNDY_NONR_1
               CALL M3EXIT(PNAME, MDATE, MTIME, XMSG, XSTAT1)
            ELSE                ! found bc's (PPM) on file
               BCON(1:NBNDY, S) = BBUF(1:NBNDY, LVL)
            END IF
         ELSE
            BCON(1:NBNDY, S) = CMIN
         END IF
      END DO

      STRT = N_GC_ADV + N_AE_ADV + N_NR_ADV + 1
      FINI = N_GC_ADV + N_AE_ADV + N_NR_ADV + N_TR_ADV
      DO S = STRT, FINI
         VNAME = BCNAME_ADV(S)
         IF ( VNAME .NE. BLNK ) THEN
            IF ( .NOT. INTERPB(BNDY_TRAC_1, VNAME, PNAME,
     &           MDATE, MTIME, NBNDY*NLAYS, BBUF) ) THEN
               XMSG = 'Could not read ' // TRIM(VNAME) //
     &              ' from ' // BNDY_TRAC_1
               CALL M3EXIT(PNAME, MDATE, MTIME, XMSG, XSTAT1)
            ELSE                ! found bc's (PPM) on file
               BCON(1:NBNDY, S) = BBUF(1:NBNDY, LVL)
            END IF
         ELSE
            BCON(1:NBNDY, S) = CMIN
         END IF
      END DO

      RETURN
      END SUBROUTINE RDBCON_IO_UNITS
