C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE UPDATE_BNDY_COND(JDATE, JTIME, STEP_NUM, XS)

C-----------------------------------------------------------------------
C Function:
C   Update the control variables in the boundary condition time series file
C   and scatter the global XS array to the local CNTL_BC_TS arrays

C Revision History:
C   Aug 2009 by Peter Percell at UH-IMAQS: Created
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      USE HGRD_DEFN             ! horizontal domain specifications
      USE FOURDVAR_MOD          ! 4DVAR data

      IMPLICIT NONE

C Include Files:
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_FILES_ID    ! file name parameters
#ifdef parallel
      INCLUDE SUBST_MPICH       ! MPI definitions and parameters
#endif

C Arguments:
      INTEGER, INTENT(IN) :: JDATE ! current date, format YYYYDDD
      INTEGER, INTENT(IN) :: JTIME ! current time, format HHMMSS
      INTEGER, INTENT(IN) :: STEP_NUM ! current output time step number
      REAL, INTENT(IN)    :: XS(1:, 1:, 1:, 0:) ! boundary condition control variables

C External Functions:
      LOGICAL, EXTERNAL :: BWRITE3, FLUSH3

C Local Variables:
      CHARACTER(16) :: PNAME = 'UPDATE_BNDY_COND'
      CHARACTER(96) :: XMSG = ' '

      INTEGER :: S              ! species index

      CHARACTER(16) :: VNAME    ! variable name

#ifdef parallel
      INTEGER :: IERROR         ! MPI error code
#endif

      INTERFACE
         SUBROUTINE SCATTER_BNDY_3D(G_ARRAY, L_ARRAY, JDATE, JTIME)
            IMPLICIT NONE
            REAL, INTENT(IN) :: G_ARRAY(:, :, :)
            REAL, INTENT(OUT) :: L_ARRAY(:, :, :)
            INTEGER, INTENT(IN) :: JDATE, JTIME
         END SUBROUTINE SCATTER_BNDY_3D
      END INTERFACE

C-----------------------------------

C Update the control variables in the boundary condition time series file
      IF ( MYPE == 0 ) THEN
         DO S = 1, N_CNTL_SPC
            VNAME = BCNAME_CNTL(S)
            IF ( .NOT. BWRITE3(ADJ_BC_TS, VNAME,
     &           JDATE, JTIME, XS(:, :, S, STEP_NUM)) ) THEN
               XMSG = 'Could not write ' // TRIM(VNAME) //
     &              ' to ' // TRIM(ADJ_BC_TS)
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
            END IF
            IF ( .NOT. FLUSH3(ADJ_BC_TS) ) THEN
               XMSG = 'Could not flush ' // TRIM(ADJ_BC_TS)
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
            END IF
         END DO
      END IF

C Scatter the global XS array to the local CNTL_BC_TS arrays
      CALL SCATTER_BNDY_3D(
     &     XS(:, :, :, STEP_NUM),
     &     CNTL_BC_TS(:, :, :, STEP_NUM),
     &     JDATE, JTIME)

#ifdef parallel
      CALL MPI_BARRIER(MPI_COMM_WORLD, IERROR)
#endif

      END SUBROUTINE UPDATE_BNDY_COND
