
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE GET_BG_FG(F)

C-----------------------------------------------------------------------
C Function:
C   Get the function value and gradient of the background operator at the
C   initial time

C Revision History:
C   Apr 2007 Kumaresh Singh and Adrian Sandu at Virginia Tech: created
C   Jun 2009 Peter Percell at UH-IMAQS: rewrote
C-----------------------------------------------------------------------

      USE HGRD_DEFN             ! horizontal domain specifications
      USE FOURDVAR_MOD          ! 4DVAR data

      IMPLICIT NONE

C Parameters:
      REAL(8), PARAMETER :: BG_WGT = 50.0D0

C Arguments:
      REAL(8), INTENT(INOUT) :: F ! cumulative value of cost function (for local domain)

C Local Variables:
      LOGICAL, SAVE :: FIRSTIME = .TRUE.

      INTEGER, SAVE :: ELAYS    ! no. of emis integration layers

      INTEGER :: C, R, L, S     ! column, row, layer and species indices

      REAL(8) :: D

C-----------------------------------------------------------------------

      IF ( FIRSTIME ) THEN
         FIRSTIME = .FALSE.

         ELAYS = SIZE(LGRID_EM_SF, 3)
      END IF

C Update the cost function F and gradient LGRID_EM_SF
      DO S = 1, N_CNTL_SPC
         DO L = 1, ELAYS
            DO R = 1, NROWS
               DO C = 1, NCOLS
                  D = (EM_SF(C, R, L, S) - 1.0) / BG_WGT
                  F = F + D**2 / 2.0D0
                  LGRID_EM_SF(C, R, L, S) =
     &                 LGRID_EM_SF(C, R, L, S) + D / BG_WGT
               END DO
            END DO
         END DO
      END DO

      END SUBROUTINE GET_BG_FG
