C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

      MODULE ADJOINT_FILES

C-----------------------------------------------------------------------
C Function:
C   Define adjoint file functions

C Revision History:
C   Aug 2007 by Peter Percell at UH-IMAQS: Created
C-----------------------------------------------------------------------

      IMPLICIT NONE
      SAVE

C-----------------------------------
C Data
C-----------------------------------

C Flags to control writing of checkpoint files, default = [F]
      LOGICAL :: WR_CHEM_CHK = .FALSE.
      LOGICAL :: WR_AFAC_CHK = .FALSE.
      LOGICAL :: WR_RHOJ_CHK = .FALSE.
      LOGICAL :: WR_XADV_CHK = .FALSE.
      LOGICAL :: WR_YADV_CHK = .FALSE.
      LOGICAL :: WR_ZADV_CHK = .FALSE.

C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

      CONTAINS

C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C Create new adjoint file
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE OP_ADJ_FILE(JDATE, JTIME, TSTEP, ADJ_FILE)

      USE HGRD_DEFN             ! horizontal domain specifications
      USE FOURDVAR_MOD          ! 4DVAR data

      IMPLICIT NONE

C Include Files:
      INCLUDE SUBST_GC_SPC      ! gas chemistry species table
      INCLUDE SUBST_AE_SPC      ! aerosol species table
      INCLUDE SUBST_NR_SPC      ! non-reactive species table
      INCLUDE SUBST_TR_SPC      ! tracer species table
      INCLUDE SUBST_GC_ADV      ! gas chem advection species and map table
      INCLUDE SUBST_AE_ADV      ! aerosol advection species and map table
      INCLUDE SUBST_NR_ADV      ! non-react advection species and map table
      INCLUDE SUBST_TR_ADV      ! tracer advection species and map table
      INCLUDE SUBST_GC_DIFF     ! gas chem diffusion species and map table
      INCLUDE SUBST_AE_DIFF     ! aerosol diffusion species and map table
      INCLUDE SUBST_NR_DIFF     ! non-react diffusion species and map table
      INCLUDE SUBST_TR_DIFF     ! tracer diffusion species and map table
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_IOFDESC     ! file header data structure
      INCLUDE SUBST_FILES_ID    ! file name parameters

C Parameters:
      INTEGER, PARAMETER :: N_SPC_ADV = N_GC_ADV
     &     + N_AE_ADV
     &     + N_NR_ADV
     &     + N_TR_ADV
     &     + 1

      REAL, PARAMETER :: CMIN = 1.0E-30
      CHARACTER(16) :: CONCMIN

C Arguments:
      INTEGER, INTENT(IN) :: JDATE ! starting date (YYYYDDD)
      INTEGER, INTENT(IN) :: JTIME ! starting time (HHMMSS)
      INTEGER, INTENT(IN) :: TSTEP(2) ! time step vector (HHMMSS)
                                !   TSTEP(1) = local output step
                                !   TSTEP(2) = sciproc sync. step (chem)
      CHARACTER(16), INTENT(IN) :: ADJ_FILE

C Local Variables:
      CHARACTER(16) :: PNAME = 'OP_ADJ_FILE'
      CHARACTER(96) :: XMSG = ' '

      INTEGER :: ALLOCSTAT

      CHARACTER(16) :: BLNK = ' '
      REAL :: BCFAC(1:N_SPC_ADV) ! multiplicative factor for BC species

      CHARACTER(16) :: VNAME    ! variable name

      INTEGER :: K, S, V        ! loop counters
      INTEGER :: STRT, FINI     ! loop limits
      INTEGER :: INDX

C-----------------------------------------------------------------------

      IF ( ADJ_FILE == ADJ_BC_SF .OR.
     &     ADJ_FILE == ADJ_LGRID_BC_SF .OR.
     &     ADJ_FILE == ADJ_BC_TS .OR.
     &     ADJ_FILE == ADJ_LGRID_BC_TS ) THEN
         IF ( .NOT. ALLOCATED(BCNAME_ADV) ) THEN
C Create map from advected species to BC species names
            ALLOCATE(
     &           BCNAME_ADV(1:N_SPC_ADV),
     &           STAT = ALLOCSTAT)
            IF ( ALLOCSTAT /= 0 ) THEN
               XMSG = 'Failure allocating memory for BCNAME_ADV'
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT2)
            END IF

            WRITE(CONCMIN, '(1PE8.2)') CMIN
            CALL ADVBC_MAP(CONCMIN, BCNAME_ADV, BCFAC)
         END IF

         IF ( .NOT. ALLOCATED(BCNAME_CNTL) ) THEN
C Create map from control species to BC species names
            ALLOCATE(
     &           BCNAME_CNTL(1:N_CNTL_SPC),
     &           STAT = ALLOCSTAT)
            IF ( ALLOCSTAT /= 0 ) THEN
               XMSG = 'Failure allocating memory for BCNAME_CNTL'
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT2)
            END IF

            DO S = 1, N_CNTL_SPC
               VNAME = BCNAME_ADV(CNTL_TO_ADV_MAP(S))
               IF ( VNAME .NE. BLNK ) THEN
                  BCNAME_CNTL(S) = VNAME
               ELSE
                  XMSG = 'The control species ' //
     &                 TRIM(CNTL_SPC_NAME(S)) //
     &                 ' must be a boundary condition species'
                  CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT3)
               END IF
            END DO
         END IF
      END IF

      IF ( MYPE == 0 ) THEN

         IF ( ADJ_FILE == ADJ_BC_SF .OR.
     &        ADJ_FILE == ADJ_LGRID_BC_SF .OR.
     &        ADJ_FILE == ADJ_BC_TS .OR.
     &        ADJ_FILE == ADJ_LGRID_BC_TS ) THEN

C Get file characteristics of BNDY_GASC_1
            IF ( .NOT. OPEN3(BNDY_GASC_1, FSREAD3, PNAME) ) THEN
               XMSG = 'Could not open ' // TRIM(BNDY_GASC_1) //
     &              ' file as OLD:READ-ONLY'
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
            END IF

            IF ( .NOT. DESC3(BNDY_GASC_1) ) THEN
               XMSG = 'Could not get ' // TRIM(BNDY_GASC_1) // ' file description'
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
            END IF

            IF ( .NOT. CLOSE3(BNDY_GASC_1) ) THEN
               XMSG = 'Could not close ' // TRIM(BNDY_GASC_1)
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
            END IF

            SDATE3D = JDATE
            STIME3D = JTIME

         ELSE

C Get file characteristics of CTM_CONC_1
            IF ( .NOT. OPEN3(CTM_CONC_1, FSREAD3, PNAME) ) THEN
               XMSG = 'Could not open ' // TRIM(CTM_CONC_1) //
     &              ' file as OLD:READ-ONLY'
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
            END IF

            IF ( .NOT. DESC3(CTM_CONC_1) ) THEN
               XMSG = 'Could not get ' // TRIM(CTM_CONC_1) // ' file description'
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
            END IF

            IF ( .NOT. CLOSE3(CTM_CONC_1) ) THEN
               XMSG = 'Could not close ' // TRIM(CTM_CONC_1)
               CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
            END IF

         END IF

C Set file characteristics that differ from the base file, BNDY_GASC_1 or
C CTM_CONC_1
         SELECT CASE ( ADJ_FILE )

         CASE ( ADJ_CHEM_CHK )

            TSTEP3D = TSTEP(2)  ! sync step

            FDESC3D(1) = 'Chemistry checkpoint output'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = N_GC_SPC

            VTYPE3D(1:N_GC_SPC) = M3REAL
            VNAME3D(1:N_GC_SPC) = GC_SPC(1:N_GC_SPC)
            UNITS3D(1:N_GC_SPC) = 'ppmV'
            VDESC3D(1:N_GC_SPC) = 'Chemistry checkpoint for ' //
     &           GC_SPC(1:N_GC_SPC)

         CASE ( ADJ_AFAC_CHK )

            TSTEP3D = TSTEP(2)  ! sync step

            FDESC3D(1) = 'Adjustment factor checkpoint output'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = 1

            VTYPE3D(1) = M3REAL
            VNAME3D(1) = 'ADJFAC'
            UNITS3D(1) = 'none'
            VDESC3D(1) = 'Concentration adjustment factor'

         CASE ( ADJ_RHOJ_CHK )

            TSTEP3D = TSTEP(2)  ! sync step

            FDESC3D(1) = 'RHOJ checkpoint output'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = 1

            VTYPE3D(1) = M3REAL
            VNAME3D(1) = 'RHOJ'
            UNITS3D(1) = 'm*kg/m**3'
            VDESC3D(1) = 'advected air density X total Jacobian'

         CASE ( ADJ_XADV_CHK )

            TSTEP3D = TSTEP(2)  ! sync step

            FDESC3D(1) = 'X1 advection checkpoint output'

            NVARS3D = N_GC_ADV + N_AE_ADV + N_NR_ADV + N_TR_ADV + 1

            STRT = 1
            FINI = N_GC_ADV
            VNAME3D(STRT:FINI) = GC_ADV(1:N_GC_ADV)

            STRT = FINI + 1
            FINI = FINI + N_AE_ADV
            VNAME3D(STRT:FINI) = AE_ADV(1:N_AE_ADV)

            STRT = FINI + 1
            FINI = FINI + N_NR_ADV
            VNAME3D(STRT:FINI) = NR_ADV(1:N_NR_ADV)

            STRT = FINI + 1
            FINI = FINI + N_TR_ADV
            VNAME3D(STRT:FINI) = TR_ADV(1:N_TR_ADV)

            VNAME3D(FINI + 1) = 'RHOJ'

            VTYPE3D(1:NVARS3D) = M3REAL
            UNITS3D(1:NVARS3D) = 'Coupled CGRID'
            VDESC3D(1:NVARS3D) = 'X1 advection checkpoint for ' //
     &           VNAME3D(1:NVARS3D)

         CASE ( ADJ_YADV_CHK )

            TSTEP3D = TSTEP(2)  ! sync step

            FDESC3D(1) = 'X2 advection checkpoint output'

            NVARS3D = N_GC_ADV + N_AE_ADV + N_NR_ADV + N_TR_ADV + 1

            STRT = 1
            FINI = N_GC_ADV
            VNAME3D(STRT:FINI) = GC_ADV(1:N_GC_ADV)

            STRT = FINI + 1
            FINI = FINI + N_AE_ADV
            VNAME3D(STRT:FINI) = AE_ADV(1:N_AE_ADV)

            STRT = FINI + 1
            FINI = FINI + N_NR_ADV
            VNAME3D(STRT:FINI) = NR_ADV(1:N_NR_ADV)

            STRT = FINI + 1
            FINI = FINI + N_TR_ADV
            VNAME3D(STRT:FINI) = TR_ADV(1:N_TR_ADV)

            VNAME3D(FINI + 1) = 'RHOJ'

            VTYPE3D(1:NVARS3D) = M3REAL
            UNITS3D(1:NVARS3D) = 'Coupled CGRID'
            VDESC3D(1:NVARS3D) = 'X2 advection checkpoint for ' //
     &           VNAME3D(1:NVARS3D)

         CASE ( ADJ_ZADV_CHK )

            TSTEP3D = TSTEP(2)  ! sync step

            FDESC3D(1) = 'X3 advection checkpoint output'

            NVARS3D = N_GC_ADV + N_AE_ADV + N_NR_ADV + N_TR_ADV + 1

            STRT = 1
            FINI = N_GC_ADV
            VNAME3D(STRT:FINI) = GC_ADV(1:N_GC_ADV)

            STRT = FINI + 1
            FINI = FINI + N_AE_ADV
            VNAME3D(STRT:FINI) = AE_ADV(1:N_AE_ADV)

            STRT = FINI + 1
            FINI = FINI + N_NR_ADV
            VNAME3D(STRT:FINI) = NR_ADV(1:N_NR_ADV)

            STRT = FINI + 1
            FINI = FINI + N_TR_ADV
            VNAME3D(STRT:FINI) = TR_ADV(1:N_TR_ADV)

            VNAME3D(FINI + 1) = 'RHOJ'

            VTYPE3D(1:NVARS3D) = M3REAL
            UNITS3D(1:NVARS3D) = 'Coupled CGRID'
            VDESC3D(1:NVARS3D) = 'X3 advection checkpoint for ' //
     &           VNAME3D(1:NVARS3D)

         CASE ( ADJ_OBS_CONC )

            TSTEP3D = TSTEP(2)  ! sync step

            FDESC3D(1) = 'Observed concentrations'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = N_OBS_SPC

            VTYPE3D(1:N_OBS_SPC) = M3REAL
            VNAME3D(1:N_OBS_SPC) = OBS_SPC_NAME(1:N_OBS_SPC)
            UNITS3D(1:N_OBS_SPC) = OBS_SPC_UNITS(1:N_OBS_SPC)
            VDESC3D(1:N_OBS_SPC) = 'Observed concentration of ' //
     &           OBS_SPC_NAME(1:N_OBS_SPC)

         CASE ( ADJ_OBS_CHK )

            TSTEP3D = TSTEP(2)  ! sync step

            FDESC3D(1) = 'Current model calculated values ' //
     &           'corresponding to observed concentrations'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = N_OBS_SPC

            VTYPE3D(1:N_OBS_SPC) = M3REAL
            VNAME3D(1:N_OBS_SPC) = OBS_SPC_NAME(1:N_OBS_SPC)
            UNITS3D(1:N_OBS_SPC) = OBS_SPC_UNITS(1:N_OBS_SPC)
            VDESC3D(1:N_OBS_SPC) = 'Current model calculated value for ' //
     &           OBS_SPC_NAME(1:N_OBS_SPC)

         CASE ( ADJ_BG_CONC )

            TSTEP3D = 0         ! time independent

            FDESC3D(1) = 'Background concentrations'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = N_CNTL_SPC

            VTYPE3D(1:N_CNTL_SPC) = M3REAL
            VNAME3D(1:N_CNTL_SPC) = CNTL_SPC_NAME(1:N_CNTL_SPC)
            UNITS3D(1:N_CNTL_SPC) = CNTL_SPC_UNITS(1:N_CNTL_SPC)
            VDESC3D(1:N_CNTL_SPC) = 'Background concentration of ' //
     &           CNTL_SPC_NAME(1:N_CNTL_SPC)

         CASE ( ADJ_CNTL_CHK )

            TSTEP3D = TSTEP(2)  ! sync step

            FDESC3D(1) = 'Control checkpoint output'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = N_CNTL_SPC

            VTYPE3D(1:N_CNTL_SPC) = M3REAL
            VNAME3D(1:N_CNTL_SPC) = CNTL_SPC_NAME(1:N_CNTL_SPC)
            UNITS3D(1:N_CNTL_SPC) = CNTL_SPC_UNITS(1:N_CNTL_SPC)
            VDESC3D(1:N_CNTL_SPC) = 'Control checkpoint for ' //
     &           CNTL_SPC_NAME(1:N_CNTL_SPC)

         CASE ( ADJ_LGRID )

            TSTEP3D = TSTEP(1)  ! conc file output step

            FDESC3D(1) = 'Sensitivity output'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = 0

C Note: "CF" stands for "cost function units"
            DO S = 1, N_GC_SPC
               NVARS3D = NVARS3D + 1
               VTYPE3D(NVARS3D) = M3REAL
               VNAME3D(NVARS3D) = GC_SPC(S)
               UNITS3D(NVARS3D) = 'CF/ppmV'
               VDESC3D(NVARS3D) = 'Sensitivity of cost function to ' //
     &              TRIM(VNAME3D(NVARS3D))
            END DO

            NVARS3D = NVARS3D + 1
            VTYPE3D(NVARS3D) = M3REAL
            VNAME3D(NVARS3D) = 'RHOJ'
            UNITS3D(NVARS3D) = 'CF/m*kg/m**3'
            VDESC3D(NVARS3D) = 'Sensitivity of cost function to ' //
     &           TRIM(VNAME3D(NVARS3D))

            DO S = 1, N_AE_SPC
               NVARS3D = NVARS3D + 1
               VTYPE3D(NVARS3D) = M3REAL
               VNAME3D(NVARS3D) = AE_SPC(S)
               IF ( VNAME3D(NVARS3D)(1:3) == 'NUM' ) THEN
                  UNITS3D(NVARS3D) = 'CF/number/m**3'
               ELSE IF ( VNAME3D(NVARS3D)(1:3) == 'SRF' ) THEN
                  UNITS3D(NVARS3D) = 'CF/m**2/m**3'
               ELSE
                  UNITS3D(NVARS3D) = 'CF/microg/m**3'
               END IF
               VDESC3D(NVARS3D) = 'Sensitivity of cost function to ' //
     &              TRIM(VNAME3D(NVARS3D))
            END DO

            DO S = 1, N_NR_SPC
               NVARS3D = NVARS3D + 1
               VTYPE3D(NVARS3D) = M3REAL
               VNAME3D(NVARS3D) = NR_SPC(S)
               UNITS3D(NVARS3D) = 'CF/ppmV'
               VDESC3D(NVARS3D) = 'Sensitivity of cost function to ' //
     &              TRIM(VNAME3D(NVARS3D))
            END DO

            DO S = 1, N_TR_SPC
               NVARS3D = NVARS3D + 1
               VTYPE3D(NVARS3D) = M3REAL
               VNAME3D(NVARS3D) = TR_SPC(S)
               UNITS3D(NVARS3D) = 'CF/ppmV'
               VDESC3D(NVARS3D) = 'Sensitivity of cost function to ' //
     &              TRIM(VNAME3D(NVARS3D))
            END DO

         CASE ( ADJ_EM_SF )

            TSTEP3D = 0         ! time independent

            FDESC3D(1) = 'Emissions Scaling Factors'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NLAYS3D = SIZE(EM_SF, 3)
            NVARS3D = N_CNTL_SPC

            VTYPE3D(1:N_CNTL_SPC) = M3REAL
            VNAME3D(1:N_CNTL_SPC) = CNTL_SPC_NAME(1:N_CNTL_SPC)
            UNITS3D(1:N_CNTL_SPC) = 'none'
            VDESC3D(1:N_CNTL_SPC) = 'Emissions Scaling Factor for ' //
     &           CNTL_SPC_NAME(1:N_CNTL_SPC)

         CASE ( ADJ_LGRID_EM_SF )

            TSTEP3D = 0         ! time independent

            FDESC3D(1) = 'Gradient of Cost Function wrt ' //
     &           'Emissions Scaling Factors'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NLAYS3D = SIZE(LGRID_EM_SF, 3)
            NVARS3D = N_CNTL_SPC

C Note: "CF" stands for "cost function units"
            VTYPE3D(1:N_CNTL_SPC) = M3REAL
            VNAME3D(1:N_CNTL_SPC) = CNTL_SPC_NAME(1:N_CNTL_SPC)
            UNITS3D(1:N_CNTL_SPC) = 'CF'
            VDESC3D(1:N_CNTL_SPC) = 'Sensitivity of cost function to ' //
     &           'Emissions scaling factor for ' // CNTL_SPC_NAME(1:N_CNTL_SPC)

         CASE ( ADJ_BC_SF )

            TSTEP3D = 0         ! time independent

            FDESC3D(1) = 'Boundary Condition Scaling Factors'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = N_CNTL_SPC

            VTYPE3D(1:N_CNTL_SPC) = M3REAL
            VNAME3D(1:N_CNTL_SPC) = CNTL_SPC_NAME(1:N_CNTL_SPC)
            UNITS3D(1:N_CNTL_SPC) = 'none'
            VDESC3D(1:N_CNTL_SPC) = 'Boundary Condition Scaling Factor for ' //
     &           CNTL_SPC_NAME(1:N_CNTL_SPC)

         CASE ( ADJ_LGRID_BC_SF )

            TSTEP3D = 0         ! time independent

            FDESC3D(1) = 'Gradient of Cost Function wrt ' //
     &           'Boundary Condition Scaling Factors'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = N_CNTL_SPC

C Note: "CF" stands for "cost function units"
            VTYPE3D(1:N_CNTL_SPC) = M3REAL
            VNAME3D(1:N_CNTL_SPC) = CNTL_SPC_NAME(1:N_CNTL_SPC)
            UNITS3D(1:N_CNTL_SPC) = 'CF'
            VDESC3D(1:N_CNTL_SPC) = 'Sensitivity of cost function to ' //
     &           'BC scaling factor for ' // CNTL_SPC_NAME(1:N_CNTL_SPC)

         CASE ( ADJ_BC_TS )

            TSTEP3D = TSTEP(1)  ! conc file output step

            FDESC3D(1) = 'Boundary Condition Time Series'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = 0

            STRT = 1
            FINI = N_GC_ADV
            DO S = STRT, FINI
               VNAME = BCNAME_ADV(S)
               IF ( VNAME .NE. BLNK ) THEN
                  NVARS3D = NVARS3D + 1
                  VTYPE3D(NVARS3D) = M3REAL
                  VNAME3D(NVARS3D) = VNAME
                  UNITS3D(NVARS3D) = 'ppmV'
                  VDESC3D(NVARS3D) = 'Boundary value for ' //
     &                 TRIM(VNAME)
               END IF
            END DO

            STRT = FINI + 1
            FINI = FINI + N_AE_ADV
            DO S = STRT, FINI
               VNAME = BCNAME_ADV(S)
               IF ( VNAME .NE. BLNK ) THEN
                  NVARS3D = NVARS3D + 1
                  VTYPE3D(NVARS3D) = M3REAL
                  VNAME3D(NVARS3D) = VNAME
                  IF ( VNAME(1:3) == 'NUM' ) THEN
                     UNITS3D(NVARS3D) = 'number/m**3'
                  ELSE IF ( VNAME(1:3) == 'SRF' ) THEN
                     UNITS3D(NVARS3D) = 'm**2/m**3'
                  ELSE
                     UNITS3D(NVARS3D) = 'microgram/m**3'
                  END IF
                  VDESC3D(NVARS3D) = 'Boundary value for ' //
     &                 TRIM(VNAME)
               END IF
            END DO

            STRT = FINI + 1
            FINI = FINI + N_NR_ADV
            DO S = STRT, FINI
               VNAME = BCNAME_ADV(S)
               IF ( VNAME .NE. BLNK ) THEN
                  NVARS3D = NVARS3D + 1
                  VTYPE3D(NVARS3D) = M3REAL
                  VNAME3D(NVARS3D) = VNAME
                  UNITS3D(NVARS3D) = 'ppmV'
                  VDESC3D(NVARS3D) = 'Boundary value for ' //
     &                 TRIM(VNAME)
               END IF
            END DO

            STRT = FINI + 1
            FINI = FINI + N_TR_ADV
            DO S = STRT, FINI
               VNAME = BCNAME_ADV(S)
               IF ( VNAME .NE. BLNK ) THEN
                  NVARS3D = NVARS3D + 1
                  VTYPE3D(NVARS3D) = M3REAL
                  VNAME3D(NVARS3D) = VNAME
                  UNITS3D(NVARS3D) = 'ppmV'
                  VDESC3D(NVARS3D) = 'Boundary value for ' //
     &                 TRIM(VNAME)
               END IF
            END DO

         CASE ( ADJ_LGRID_BC_TS )

            TSTEP3D = TSTEP(1)  ! conc file output step

            FDESC3D(1) = 'Gradient of Cost Function wrt ' //
     &           'Boundary Condition Time Series'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = N_CNTL_SPC

C Note: "CF" stands for "cost function units"
            VTYPE3D(1:N_CNTL_SPC) = M3REAL
            VNAME3D(1:N_CNTL_SPC) = BCNAME_CNTL(1:N_CNTL_SPC)
            UNITS3D(1:N_CNTL_SPC) = 'CF/' // CNTL_SPC_UNITS(1:N_CNTL_SPC)
            VDESC3D(1:N_CNTL_SPC) = 'Sensitivity of cost function to ' //
     &           'BC time series for ' // BCNAME_CNTL(1:N_CNTL_SPC)

         CASE ( ADJ_EMGRID )

C TODO: FIGURE OUT WHAT IS REALLY NEEDED HERE.
            TSTEP3D = TSTEP(1)  ! conc file output step

            FDESC3D(1) = 'Emission sensitivity output'
            FDESC3D(2) = 'From CMAQ model dyn alloc version CTM'
            DO K = 3, MXDESC3
               FDESC3D(K) = ' '
            END DO

            NVARS3D = N_GC_DIFF + N_AE_DIFF + N_NR_DIFF + N_TR_DIFF

            V = 0
            STRT = 1
            FINI = N_GC_DIFF
            DO S = STRT, FINI
               V = V + 1
               INDX = GC_DIFF_MAP(V)
               VTYPE3D(S) = M3REAL
               VNAME3D(S) = GC_SPC(INDX)
               UNITS3D(S) = 'ppmV'
               VDESC3D(S) = 'Variable ' // VNAME3D(S)
            END DO

            V = 0
            STRT = FINI + 1     ! STRT = N_GC_DIFF + 1
            FINI = N_GC_DIFF + N_AE_DIFF
            DO S = STRT, FINI
               V = V + 1
               INDX = AE_DIFF_MAP(V)
               VTYPE3D(S) = M3REAL
               VNAME3D(S) = AE_SPC(INDX) ! from include file
               IF ( VNAME3D(S)(1:3) == 'NUM' ) THEN
                  UNITS3D(S) = 'number/m**3'
               ELSE IF ( VNAME3D(S)(1:3) == 'SRF' ) THEN
                  UNITS3D(S) = 'm**2/m**3'
               ELSE
                  UNITS3D(S) = 'micrograms/m**3'
               END IF
               VDESC3D(S) = 'Variable ' // VNAME3D(S)
            END DO

            V = 0
            STRT = FINI + 1     ! STRT = N_GC_DIFF + N_AE_DIFF + 1
            FINI = N_GC_DIFF + N_AE_DIFF + N_NR_SPC ! write all NR species
            DO S = STRT, FINI
               V = V + 1
               VTYPE3D(S) = M3REAL
               VNAME3D(S) = NR_SPC(V) ! from include file
               UNITS3D(S) = 'ppmV'
               VDESC3D(S) = 'Variable ' // VNAME3D(S)
            END DO

            V = 0
            STRT = FINI + 1     ! STRT = N_GC_DIFF + N_AE_DIFF + N_NR_SPC + 1
            FINI = N_GC_DIFF + N_AE_DIFF + N_NR_SPC + N_TR_SPC ! write all TR species
            DO S = STRT, FINI
               V = V + 1
               VTYPE3D(S) = M3REAL
               VNAME3D(S) = TR_SPC(V) ! from include file
               UNITS3D(S) = 'ppmV'
               VDESC3D(S) = 'Variable ' // VNAME3D(S)
            END DO

         CASE DEFAULT

            XMSG =  '"' // TRIM(ADJ_FILE) // '"' //
     &           ' is not a valid file selection'
            CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT3)

         END SELECT

C Create new file
         IF ( .NOT. OPEN3(ADJ_FILE, FSCREA3, PNAME) ) THEN
            XMSG = 'Could not create ' // TRIM(ADJ_FILE) // ' file'
            CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
         END IF

      END IF                    ! MYPE == 0

      SELECT CASE ( ADJ_FILE )
      CASE ( ADJ_CHEM_CHK )
         WR_CHEM_CHK = .TRUE.
      CASE ( ADJ_AFAC_CHK )
         WR_AFAC_CHK = .TRUE.
      CASE ( ADJ_RHOJ_CHK )
         WR_RHOJ_CHK = .TRUE.
      CASE ( ADJ_XADV_CHK )
         WR_XADV_CHK = .TRUE.
      CASE ( ADJ_YADV_CHK )
         WR_YADV_CHK = .TRUE.
      CASE ( ADJ_ZADV_CHK )
         WR_ZADV_CHK = .TRUE.
      END SELECT

      END SUBROUTINE OP_ADJ_FILE

C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C Write to an adjoint file
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE WR_ADJ_FILE(ARRAY, JDATE, JTIME, ADJ_FILE)

      IMPLICIT NONE

C Include Files:
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
#include      SUBST_IODECL      # I/O definitions and declarations
      INCLUDE SUBST_FILES_ID    ! file name parameters

C Arguments:
      REAL, INTENT(IN) :: ARRAY(*)
      INTEGER, INTENT(IN) :: JDATE ! current model date, coded YYYYDDD
      INTEGER, INTENT(IN) :: JTIME ! current model time, coded HHMMSS
      CHARACTER(16), INTENT(IN) :: ADJ_FILE

C Local variables:
      LOGICAL, SAVE :: FIRSTIME = .TRUE.
      INTEGER, SAVE :: LOGDEV

      CHARACTER(16) :: PNAME = 'WR_ADJ_FILE'
      CHARACTER(96) :: XMSG = ' '

C-----------------------------------------------------------------------

      IF ( FIRSTIME ) THEN
         FIRSTIME = .FALSE.
         LOGDEV = INIT3()
      END IF

      IF ( ADJ_FILE == ADJ_BC_SF .OR.
     &     ADJ_FILE == ADJ_LGRID_BC_SF .OR.
     &     ADJ_FILE == ADJ_BC_TS .OR.
     &     ADJ_FILE == ADJ_LGRID_BC_TS ) THEN
         XMSG = 'Wrong subroutine for writing to ' // TRIM(ADJ_FILE)
         CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT3)
      END IF

      IF ( .NOT. WRITE3(ADJ_FILE, ALLVAR3, JDATE, JTIME, ARRAY) ) THEN
         XMSG = 'Could not write to ' // TRIM(ADJ_FILE)
         CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
      END IF

      WRITE(LOGDEV, '(/5X, 3(A, :, 1X), I8, ":", I6.6)')
     &     'Timestep written to', ADJ_FILE,
     &     'for date and time', JDATE, JTIME

      END SUBROUTINE WR_ADJ_FILE

C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C Read from an adjoint file
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE RD_ADJ_FILE(ARRAY, JDATE, JTIME, ADJ_FILE)

      USE GRID_CONF             ! horizontal & vertical domain specifications

      IMPLICIT NONE

C Include Files:
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_IOFDESC     ! file header data structure
      INCLUDE SUBST_FILES_ID    ! file name parameters

C Arguments:
      REAL, INTENT(OUT) :: ARRAY(*)
      INTEGER, INTENT(IN) :: JDATE ! current model date, coded YYYYDDD
      INTEGER, INTENT(IN) :: JTIME ! current model time, coded HHMMSS
      CHARACTER(16), INTENT(IN) :: ADJ_FILE

C Local variables:
      LOGICAL, SAVE :: FIRSTIME = .TRUE.

      CHARACTER(16) :: PNAME = 'RD_ADJ_FILE'
      CHARACTER(96) :: XMSG = ' '

C Note: Prefix GL_: with respect to global domain
C       Suffix _Z:  zero location
C for sub-domain associated with MYPE (MY Processing Element)
      INTEGER, SAVE :: GL_COL_Z, GL_ROW_Z

      REAL :: DBUFF(1:GL_NCOLS, 1:GL_NROWS, 1:NLAYS)

      INTEGER :: K, C, R, L, V  ! loop counters

C-----------------------------------------------------------------------

      IF ( FIRSTIME ) THEN
         FIRSTIME = .FALSE.

C Get local domain window in global domain
         GL_COL_Z = COLSX_PE(1, MYPE+1) - 1
         GL_ROW_Z = ROWSX_PE(1, MYPE+1) - 1
      END IF

      IF ( ADJ_FILE == ADJ_BC_SF .OR.
     &     ADJ_FILE == ADJ_LGRID_BC_SF .OR.
     &     ADJ_FILE == ADJ_BC_TS .OR.
     &     ADJ_FILE == ADJ_LGRID_BC_TS ) THEN
         XMSG = 'Wrong subroutine for reading from ' // TRIM(ADJ_FILE)
         CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT3)
      END IF

C Get file characteristics of ADJ_FILE
      IF ( .NOT. DESC3(ADJ_FILE) ) THEN
         XMSG = 'Could not get ' // TRIM(ADJ_FILE) // ' file description'
         CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
      END IF

      DO V = 1, NVARS3D
         IF ( .NOT. READ3(ADJ_FILE, VNAME3D(V), ALLAYS3,
     &        JDATE, JTIME, DBUFF) ) THEN
            XMSG = 'Could not read ' // TRIM(VNAME3D(V)) //
     &           ' from ' // ADJ_FILE
            CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
         END IF

         K = (V - 1) * NCOLS * NROWS * NLAYS
         DO L = 1, NLAYS
            DO R = GL_ROW_Z + 1, GL_ROW_Z + NROWS
               DO C = GL_COL_Z + 1, GL_COL_Z + NCOLS
                  K = K + 1
                  ARRAY(K) = DBUFF(C, R, L)
               END DO
            END DO
         END DO
      END DO

      END SUBROUTINE RD_ADJ_FILE

C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      END MODULE ADJOINT_FILES
