C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE SCATTER_BNDY_3D(G_ARRAY, L_ARRAY, JDATE, JTIME)

C-----------------------------------------------------------------------
C Function:
C   Scatter a gridded 4D global array from processor 0 to gridded 4D arrays
C   on local processors

C Revision History:
C   Jul 2009 Peter Percell at UH-IMAQS: created
C-----------------------------------------------------------------------

      USE GRID_CONF             ! horizontal & vertical domain specifications

      IMPLICIT NONE

C Includes:
#ifdef parallel
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions (XSTAT*)
      INCLUDE SUBST_MPICH       ! MPI definitions and parameters
#endif

C Arguments:
      REAL, INTENT(IN) :: G_ARRAY(:, :, :) ! global 4D array
      REAL, INTENT(OUT) :: L_ARRAY(:, :, :) ! local 4D array
      INTEGER, INTENT(IN) :: JDATE ! date, coded YYYYDDD
      INTEGER, INTENT(IN) :: JTIME ! time, coded HHMMSS

C Local Variables:
#ifdef parallel
      LOGICAL, SAVE :: FIRSTIME = .TRUE.

      CHARACTER(16) :: PNAME = 'SCATTER_BNDY_3D'
      CHARACTER(96) :: XMSG = ' '

      LOGICAL, SAVE :: S_BND_PE, E_BND_PE, N_BND_PE, W_BND_PE

      INTEGER, SAVE :: SZ, GL_SZ ! local, global zero index for southern boundary
      INTEGER, SAVE :: EZ, GL_EZ ! local, global zero index for eastern boundary
      INTEGER, SAVE :: NZ, GL_NZ ! local, global zero index for northern boundary
      INTEGER, SAVE :: WZ, GL_WZ ! local, global zero index for western boundary

      INTEGER :: GL_COLZ, GL_ROWZ

      INTEGER :: IERROR         ! MPI error code
#endif

C-----------------------------------------------------------------------

#ifdef parallel

      IF ( FIRSTIME ) THEN
         FIRSTIME = .FALSE.

         S_BND_PE = ( ROWSX_PE(1, MYPE+1) == 1 )
         E_BND_PE = ( COLSX_PE(2, MYPE+1) == GL_NCOLS )
         N_BND_PE = ( ROWSX_PE(2, MYPE+1) == GL_NROWS )
         W_BND_PE = ( COLSX_PE(1, MYPE+1) == 1 )

         SZ = 0
         EZ = NCOLS + 1
         NZ = NCOLS + NROWS + 3
         WZ = 2 * NCOLS + NROWS + 4

         GL_COLZ = COLSX_PE(1, MYPE+1) - 1
         GL_ROWZ = ROWSX_PE(1, MYPE+1) - 1

         GL_SZ = GL_COLZ
         GL_EZ = GL_ROWZ + GL_NCOLS + 1
         GL_NZ = GL_COLZ + GL_NCOLS + GL_NROWS + 3
         GL_WZ = GL_ROWZ + 2 * GL_NCOLS + GL_NROWS + 4
      END IF

C Broadcast the global array to all processors
      CALL MPI_BCAST(G_ARRAY, SIZE(G_ARRAY), MPI_REAL, 0,
     &     MPI_COMM_WORLD, IERROR)

      IF ( IERROR /= 0 ) THEN
         XMSG = 'Failed to broadcast global array from processor 0'
         CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT3)
      END IF

C South boundary
      IF ( S_BND_PE ) L_ARRAY(SZ + 1 : SZ + NCOLS + 1, :, :) =
     &     G_ARRAY(GL_SZ + 1 : GL_SZ + NCOLS + 1, :, :)

C East boundary
      IF ( E_BND_PE ) L_ARRAY(EZ + 1 : EZ + NROWS + 1, :, :) =
     &     G_ARRAY(GL_EZ + 1 : GL_EZ + NROWS + 1, :, :)

C North boundary
      IF ( N_BND_PE ) L_ARRAY(NZ : NZ + NCOLS, :, :) =
     &     G_ARRAY(GL_NZ : GL_NZ + NCOLS, :, :)

C West boundary
      IF ( W_BND_PE ) L_ARRAY(WZ : WZ + NROWS, :, :) =
     &     G_ARRAY(GL_WZ : GL_WZ + NROWS, :, :)

#else

      L_ARRAY(1:NBNDY, :, :) = G_ARRAY(1:NBNDY, :, :)

#endif

      END SUBROUTINE SCATTER_BNDY_3D
