C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE SCATTER_GRDD_4D(G_ARRAY, L_ARRAY, JDATE, JTIME)

C-----------------------------------------------------------------------
C Function:
C   Scatter a gridded 4D global array from processor 0 to gridded 4D arrays
C   on local processors

C Revision History:
C   Jul 2009 Peter Percell at UH-IMAQS: created
C-----------------------------------------------------------------------

      USE GRID_CONF             ! horizontal & vertical domain specifications

      IMPLICIT NONE

C Includes:
#ifdef parallel
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions (XSTAT*)
      INCLUDE SUBST_MPICH       ! MPI definitions and parameters
#endif

C Arguments:
      REAL, INTENT(IN) :: G_ARRAY(:, :, :, :) ! global 4D array
      REAL, INTENT(OUT) :: L_ARRAY(:, :, :, :) ! local 4D array
      INTEGER, INTENT(IN) :: JDATE ! date, coded YYYYDDD
      INTEGER, INTENT(IN) :: JTIME ! time, coded HHMMSS

C Local Variables:
#ifdef parallel
      CHARACTER(16) :: PNAME = 'SCATTER_GRDD_4D'
      CHARACTER(96) :: XMSG = ' '

      INTEGER :: IERROR         ! MPI error code
#endif

C-----------------------------------------------------------------------

#ifdef parallel
C Broadcast the global array to all processors
      CALL MPI_BCAST(G_ARRAY, SIZE(G_ARRAY), MPI_REAL, 0,
     &     MPI_COMM_WORLD, IERROR)

      IF ( IERROR /= 0 ) THEN
         XMSG = 'Failed to broadcast global array from processor 0'
         CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT3)
      END IF
#endif

C Copy the local array from the appropriate section of the global array
      L_ARRAY(1:NCOLS, 1:NROWS, :, :) = G_ARRAY(
     &     COLSX_PE(1, MYPE+1) : COLSX_PE(2, MYPE+1),
     &     ROWSX_PE(1, MYPE+1) : ROWSX_PE(2, MYPE+1),
     &     :, :)

      END SUBROUTINE SCATTER_GRDD_4D
