C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C Set input environment variables to point to files for the current day
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE SET_INPUT_ENV(JDATE, JTIME, DAY_NUM)

      IMPLICIT NONE

C Include Files:
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_IOFDESC     ! file header data structure
      INCLUDE SUBST_FILES_ID    ! file name parameters

C Arguments:
      INTEGER, INTENT(IN) :: JDATE ! current model date, coded YYYYDDD
      INTEGER, INTENT(IN) :: JTIME ! current model time, coded HHMMSS
      INTEGER, INTENT(IN) :: DAY_NUM ! current day number (starting with 1)

C External Functions (not declared in IODECL3.EXT):
      INTEGER, EXTERNAL :: ENVINT

C Local Variables:
      LOGICAL, SAVE :: FIRSTIME = .TRUE.

      INTEGER, SAVE :: NDAYS    ! number of days for run
      CHARACTER(80) :: VARDESC  ! environment variable description
      INTEGER :: STATUS         ! ENVINT status

      CHARACTER(16) :: PNAME = 'SET_INPUT_ENV'
      CHARACTER(96) :: XMSG = ' '

      CHARACTER(2) :: CDAY_NUM

      CHARACTER(512) :: EQNAME  ! value of an environment variable
      CHARACTER(512) :: PATH_BC_TS ! full path to ADJ_BC_TS file

      LOGICAL :: IERROR         ! cumulative error flag for RESET_ENV_VAR calls

C-----------------------------------

      IF ( FIRSTIME ) THEN
         FIRSTIME = .FALSE.

         NDAYS = 1              ! default
         VARDESC = 'Number of days for run'
         NDAYS = ENVINT('ADJ_NDAYS', VARDESC, NDAYS, STATUS)
         IF ( STATUS /= 0 .OR. NDAYS < 1 ) THEN
            XMSG = VARDESC
            CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT2)
         END IF
      END IF

C Set input environment variables to point to files for the current day
      IF ( DAY_NUM > NDAYS ) THEN ! use last day's input files after last day
         WRITE(CDAY_NUM, '(I2.2)') NDAYS
      ELSE
         WRITE(CDAY_NUM, '(I2.2)') DAY_NUM
      END IF

      CALL NAMEVAL('XJ_DATA_' // CDAY_NUM, EQNAME)
      CALL SETENVVAR(XJ_DATA, EQNAME)

      CALL NAMEVAL('OBS_DATA_' // CDAY_NUM, EQNAME)
      CALL SETENVVAR(OBS_DATA, EQNAME)

      IERROR = .FALSE.

C The file ADJ_BC_TS is created by combining the boundary condition files for
C the different species classes into one file and concatenating the combined
C files in time, so it contains all the boundary conditions needed for the run
      CALL NAMEVAL(ADJ_BC_TS, PATH_BC_TS)

      CALL NAMEVAL(BNDY_GASC_1, EQNAME)
      IF ( EQNAME /= PATH_BC_TS ) THEN
         IF ( .NOT. RESET_ENV_VAR(BNDY_GASC_1, 'BNDY_GASC_') ) IERROR = .TRUE.
      END IF

      CALL NAMEVAL(BNDY_AERO_1, EQNAME)
      IF ( EQNAME /= PATH_BC_TS ) THEN
         IF ( .NOT. RESET_ENV_VAR(BNDY_AERO_1, 'BNDY_AERO_') ) IERROR = .TRUE.
      END IF

      CALL NAMEVAL(BNDY_NONR_1, EQNAME)
      IF ( EQNAME /= PATH_BC_TS ) THEN
         IF ( .NOT. RESET_ENV_VAR(BNDY_NONR_1, 'BNDY_NONR_') ) IERROR = .TRUE.
      END IF

      CALL NAMEVAL(BNDY_TRAC_1, EQNAME)
      IF ( EQNAME /= PATH_BC_TS ) THEN
         IF ( .NOT. RESET_ENV_VAR(BNDY_TRAC_1, 'BNDY_TRAC_') ) IERROR = .TRUE.
      END IF

      IF ( .NOT. RESET_ENV_VAR(EMIS_1, 'EMIS_') ) IERROR = .TRUE.
      IF ( .NOT. RESET_ENV_VAR(GRID_DOT_2D, 'GRID_DOT_2D_') ) IERROR = .TRUE.
      IF ( .NOT. RESET_ENV_VAR(GRID_CRO_2D, 'GRID_CRO_2D_') ) IERROR = .TRUE.
      IF ( .NOT. RESET_ENV_VAR(MET_CRO_2D, 'MET_CRO_2D_') ) IERROR = .TRUE.
      IF ( .NOT. RESET_ENV_VAR(MET_DOT_3D, 'MET_DOT_3D_') ) IERROR = .TRUE.
      IF ( .NOT. RESET_ENV_VAR(MET_CRO_3D, 'MET_CRO_3D_') ) IERROR = .TRUE.
      IF ( .NOT. RESET_ENV_VAR(MET_BDY_3D, 'MET_BDY_3D_') ) IERROR = .TRUE.
      IF ( .NOT. RESET_ENV_VAR(DEPV_TRAC_1, 'DEPV_TRAC_') ) IERROR = .TRUE.

      IF ( IERROR ) THEN
         XMSG = 'Fatal error resetting input environment variable'
         CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT3)
      END IF

      RETURN

C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

      CONTAINS

C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      FUNCTION RESET_ENV_VAR(FNAME, FPREFIX)

      IMPLICIT NONE

C Function Value:
      LOGICAL :: RESET_ENV_VAR

C Arguments:
      CHARACTER(16), INTENT(IN) :: FNAME ! logical file name
      CHARACTER*(*), INTENT(IN) :: FPREFIX ! prefix for daily file name

C Local Variables:
      CHARACTER(16) :: PNAME = 'RESET_ENV_VAR'

      INTEGER :: FTYPE_P        ! file type
      INTEGER :: TSTEP_P        ! file time step   HHMMSS
      INTEGER :: NVARS_P        ! number of species
      INTEGER :: NCOLS_P        ! number of grid columns
      INTEGER :: NROWS_P        ! number of grid rows
      INTEGER :: NLAYS_P        ! number of layers
      INTEGER :: NTHIK_P        ! BOUNDARY:  perim thickness (cells)
                                ! SPARSE MATRIX:  number of matrix-cols
      INTEGER :: GDTYP_P        ! grid type:  1=LAT-LON, 2=Lambert, ...

      REAL(8) :: P_ALP_P        ! first, second, third map
      REAL(8) :: P_BET_P        ! projection descriptive
      REAL(8) :: P_GAM_P        ! parameters.

      REAL(8) :: XCENT_P        ! lon for coord-system X=0
      REAL(8) :: YCENT_P        ! lat for coord-system Y=0
      REAL(8) :: XORIG_P        ! X-coordinate origin of grid (map units)
      REAL(8) :: YORIG_P        ! Y-coordinate origin of grid
      REAL(8) :: XCELL_P        ! X-coordinate cell dimension
      REAL(8) :: YCELL_P        ! Y-coordinate cell dimension

      INTEGER :: VGTYP_P        ! vertical coordinate type (VGSIGP3, ...)
      REAL :: VGTOP_P           ! model-top, for sigma coord types.
      REAL :: VGLVS_P(MXLAYS3 + 1) ! vertical coord values.

      CHARACTER(16) :: GDNAM_P  ! grid name             (length NAMLEN3=16)

      INTEGER :: VTYPE_P(MXVARS3) ! variable type:  M3(INT|REAL|DBLE)
      CHARACTER(16) :: VNAME_P(MXVARS3) ! variable names (length MXDLEN3=80)
      CHARACTER(16) :: UNITS_P(MXVARS3) !   "   units or 'none' (MXDLEN3=80)
      CHARACTER(80) :: VDESC_P(MXVARS3) !   "      descriptions (MXDLEN3=80)

      INTEGER :: K              ! loop counter

C-----------------------------------

      RESET_ENV_VAR = .TRUE.

      IF ( OPEN3(FNAME, FSREAD3, PNAME) ) THEN ! OPEN3 previous version of file
         IF ( DESC3(FNAME) ) THEN ! DESC3 previous version of file
C Save file description data for previous version of file
            FTYPE_P = FTYPE3D
            TSTEP_P = TSTEP3D
            NVARS_P = NVARS3D
            NCOLS_P = NCOLS3D
            NROWS_P = NROWS3D
            NLAYS_P = NLAYS3D
            NTHIK_P = NTHIK3D
            GDTYP_P = GDTYP3D

            P_ALP_P = P_ALP3D
            P_BET_P = P_BET3D
            P_GAM_P = P_GAM3D

            XCENT_P = XCENT3D
            YCENT_P = YCENT3D
            XORIG_P = XORIG3D
            YORIG_P = YORIG3D
            XCELL_P = XCELL3D
            YCELL_P = YCELL3D

            VGTYP_P = VGTYP3D
            VGTOP_P = VGTOP3D
            VGLVS_P(:) = VGLVS3D(:)

            GDNAM_P = GDNAM3D

            VTYPE_P(:) = VTYPE3D(:)
            VNAME_P(:) = VNAME3D(:)
            UNITS_P(:) = UNITS3D(:)
            VDESC_P(:) = VDESC3D(:)
         ELSE
            RESET_ENV_VAR = .FALSE.
            XMSG = 'DESC3 failed for ' // TRIM(FNAME)
            CALL M3MESG(XMSG)
         END IF                 ! DESC3 previous version of file

C Close input file; free the associated unit number for re-use
         IF ( .NOT. CLOSE3(FNAME) ) THEN
            RESET_ENV_VAR = .FALSE.
            XMSG = 'Could not close ' // TRIM(FNAME)
            CALL M3WARN(PNAME, JDATE, JTIME, XMSG)
         END IF

         CALL NAMEVAL(TRIM(FPREFIX) // CDAY_NUM, EQNAME)
         CALL SETENVVAR(FNAME, EQNAME)

         IF ( OPEN3(FNAME, FSREAD3, PNAME) ) THEN ! OPEN3 new version of file
            IF ( DESC3(FNAME) ) THEN ! DESC3 new version of file
C Verify that the file description data for the new version of the file is
C consistent with the previous version of the file
               IF ( FTYPE3D /= FTYPE_P .OR.
     &              TSTEP3D /= TSTEP_P .OR.
     &              NVARS3D /= NVARS_P .OR.
     &              NCOLS3D /= NCOLS_P .OR.
     &              NROWS3D /= NROWS_P .OR.
     &              NLAYS3D /= NLAYS_P .OR.
     &              NTHIK3D /= NTHIK_P .OR.
     &              GDTYP3D /= GDTYP_P .OR.
     &              P_ALP3D /= P_ALP_P .OR.
     &              P_BET3D /= P_BET_P .OR.
     &              P_GAM3D /= P_GAM_P .OR.
     &              XCENT3D /= XCENT_P .OR.
     &              YCENT3D /= YCENT_P .OR.
     &              XORIG3D /= XORIG_P .OR.
     &              YORIG3D /= YORIG_P .OR.
     &              XCELL3D /= XCELL_P .OR.
     &              YCELL3D /= YCELL_P .OR.
     &              VGTYP3D /= VGTYP_P .OR.
     &              VGTOP3D /= VGTOP_P .OR.
     &              GDNAM3D /= GDNAM_P ) THEN
                  RESET_ENV_VAR = .FALSE.
                  XMSG = 'Inconsistent scalar data for new version of ' //
     &                 TRIM(FNAME)
                  CALL M3MESG(XMSG)
               END IF

               DO K = 1, MXLAYS3 + 1
                  IF ( VGLVS3D(K) /= VGLVS_P(K) ) THEN
                     RESET_ENV_VAR = .FALSE.
                     XMSG = 'Inconsistent VGLVS3D data for new version of ' //
     &                    TRIM(FNAME)
                     CALL M3MESG(XMSG)
                  END IF
               END DO

               DO K = 1, MXVARS3
                  IF ( VTYPE3D(K) /= VTYPE_P(K) .OR.
     &                 VNAME3D(K) /= VNAME_P(K) .OR.
     &                 UNITS3D(K) /= UNITS_P(K) .OR.
     &                 VDESC3D(K) /= VDESC_P(K) ) THEN
                     RESET_ENV_VAR = .FALSE.
                     XMSG = 'Inconsistent array data for new version of ' //
     &                    TRIM(FNAME)
                     CALL M3MESG(XMSG)
                  END IF
               END DO
            ELSE
               RESET_ENV_VAR = .FALSE.
               XMSG = 'DESC3 failed for ' // TRIM(FNAME)
               CALL M3MESG(XMSG)
            END IF              ! DESC3 new version of file
         END IF                 ! OPEN3 new version of file
      END IF                    ! OPEN3 previous version of file

      END FUNCTION RESET_ENV_VAR

C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      END SUBROUTINE SET_INPUT_ENV
