
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE DRIVER_FWD(CGRID, JDATE, JTIME, TSTEP, NSTEPS, NREPS,
     &     DAY_NUM, XFIRST)

C-----------------------------------------------------------------------
C Function:
C   Do forward simulation and write checkpoint states

C Revision History:
C   Apr 2007 Kumaresh Singh and Adrian Sandu at Virginia Tech: created
C   Aug 2007 Peter Percell at UH-IMAQS: rewrote
C-----------------------------------------------------------------------

      USE GRID_CONF             ! horizontal & vertical domain specifications
      USE ADJOINT_FILES         ! adjoint file data and functions

      IMPLICIT NONE

C Include Files:
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_FILES_ID    ! file name parameters

C Arguments:
      REAL, POINTER :: CGRID(:, :, :, :)
      INTEGER, INTENT(INOUT) :: JDATE ! current model date, coded YYYYDDD
      INTEGER, INTENT(INOUT) :: JTIME ! current model time, coded HHMMSS
      INTEGER, INTENT(IN) :: TSTEP(2) ! time step vector (HHMMSS)
                                      !   TSTEP(1) = local output step
                                      !   TSTEP(2) = sciproc sync. step (chem)
      INTEGER, INTENT(IN) :: NSTEPS   ! run duration: number of output time steps
      INTEGER, INTENT(IN) :: NREPS    ! number of model time steps per output step
      INTEGER, INTENT(OUT) :: DAY_NUM ! day number
      LOGICAL, INTENT(INOUT) :: XFIRST

C Local Variables:
      LOGICAL, SAVE :: FIRSTIME = .TRUE.

      CHARACTER(16) :: PNAME = 'DRIVER_FWD'
      CHARACTER(96) :: XMSG = ' '

      INTEGER, ALLOCATABLE, SAVE :: NADVSS(:) ! number of advection sub-steps per sync step
      INTEGER :: ALLOCSTAT

      INTEGER :: STEP_NUM       ! current output time step number
      INTEGER :: IREP           ! model step number within this output step

      INTEGER :: SDATE          ! date of current input files (YYYYDDD)

      INTERFACE
         SUBROUTINE ADVSTEP(JDATE, JTIME, TSTEP, NADVSS)
            IMPLICIT NONE
            INTEGER, INTENT(IN)  :: JDATE, JTIME
            INTEGER, INTENT(IN)  :: TSTEP(2)
            INTEGER, INTENT(OUT) :: NADVSS(:)
         END SUBROUTINE ADVSTEP
         SUBROUTINE SCIPROC(CGRID, JDATE, JTIME, TSTEP, NADVSS, XFIRST)
            IMPLICIT NONE
            REAL, POINTER          :: CGRID(:, :, :, :)
            INTEGER, INTENT(INOUT) :: JDATE, JTIME
            INTEGER, INTENT(IN)    :: TSTEP(2)
            INTEGER, INTENT(IN)    :: NADVSS(:)
            LOGICAL, INTENT(INOUT) :: XFIRST
         END SUBROUTINE SCIPROC
         SUBROUTINE WR_CONC(CGRID, JDATE, JTIME)
            IMPLICIT NONE
            REAL, POINTER       :: CGRID(:, :, :, :)
            INTEGER, INTENT(IN) :: JDATE, JTIME
         END SUBROUTINE WR_CONC
      END INTERFACE

C-----------------------------------------------------------------------

      IF ( FIRSTIME ) THEN
         FIRSTIME = .FALSE.

         ALLOCATE(
     &        NADVSS(1:NLAYS),
     &        STAT = ALLOCSTAT)
         IF ( ALLOCSTAT /= 0 ) THEN
            XMSG = 'Failure allocating NADVSS'
            CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT1)
         END IF
      END IF                    ! FIRSTIME

C Set input environment variables to point to files for the current day
      DAY_NUM = 1
      CALL SET_INPUT_ENV(JDATE, JTIME, DAY_NUM)
      SDATE = JDATE

C Do forward simulation and write checkpoint states
      DO STEP_NUM = 1, NSTEPS
C Get number of advection sub-steps per sync step, NADVSS(L)
         CALL ADVSTEP(JDATE, JTIME, TSTEP, NADVSS)

         DO IREP = 1, NREPS
            CALL SCIPROC(CGRID, JDATE, JTIME, TSTEP, NADVSS, XFIRST) ! science processes
         END DO

         CALL WR_CONC(CGRID, JDATE, JTIME)

         IF ( JDATE > SDATE ) THEN
            DAY_NUM = DAY_NUM + 1
            CALL SET_INPUT_ENV(JDATE, JTIME, DAY_NUM)
            SDATE = JDATE
         END IF
      END DO

      END SUBROUTINE DRIVER_FWD
