
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE SCIPROC_ADJ(LGRID, EMGRID, JDATE, JTIME, TSTEP, NADVSS, XFIRST)

C-----------------------------------------------------------------------
C Function:
C   Science process adjoint for sensitivity analysis

C Revision History:
C   Apr 2007 Kumaresh Singh and Adrian Sandu at Virginia Tech: created
C   Aug 2007 Peter Percell at UH-IMAQS: rewrote
C-----------------------------------------------------------------------

      USE GRID_CONF             ! horizontal & vertical domain specifications

      IMPLICIT NONE

C Include files:
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations

C Arguments:
      REAL, POINTER :: LGRID(:,:,:,:)  ! adjoint variable
      REAL, POINTER :: EMGRID(:,:,:,:) ! emission gradient variable
      INTEGER, INTENT(INOUT) :: JDATE ! current model date, coded YYYYDDD
      INTEGER, INTENT(INOUT) :: JTIME ! current model time, coded HHMMSS
      INTEGER, INTENT(IN) :: TSTEP(2) ! time step vector (HHMMSS)
                                      !   TSTEP(1) = local output step
                                      !   TSTEP(2) = sciproc sync. step (chem)
      INTEGER, INTENT(IN) :: NADVSS(:) ! number of advection sub-steps per sync step
      LOGICAL, INTENT(INOUT) :: XFIRST

C Local Variables:
      LOGICAL, SAVE :: FIRSTIME = .TRUE.
      INTEGER, SAVE :: LOGDEV

      CHARACTER(16) :: PNAME = 'SCIPROC_ADJ'
      CHARACTER(96) :: XMSG = ' '
      CHARACTER(36) :: NMSG = 'After NEXTIME: returned JDATE, JTIME'

      INTEGER :: ALLOCSTAT

      INTEGER :: SDATE          ! current science process date, coded YYYYDDD
      INTEGER :: STIME          ! current science process time, coded HHMMSS

      INTERFACE
         SUBROUTINE CHEM_ADJ(LGRID, JDATE, JTIME, TSTEP)
            IMPLICIT NONE
            REAL, POINTER       :: LGRID(:,:,:,:)
            INTEGER, INTENT(IN) :: JDATE, JTIME
            INTEGER, INTENT(IN) :: TSTEP(2)
         END SUBROUTINE CHEM_ADJ
         SUBROUTINE HDIFF_AD(LGRID, JDATE, JTIME, TSTEP)
            IMPLICIT NONE
            REAL, POINTER       :: LGRID(:,:,:,:)
            INTEGER, INTENT(IN) :: JDATE, JTIME
            INTEGER, INTENT(IN) :: TSTEP(2)
         END SUBROUTINE HDIFF_AD
         SUBROUTINE ZADV_CAD(LGRID, JDATE, JTIME, TSTEP)
            IMPLICIT NONE
            REAL, POINTER       :: LGRID(:,:,:,:)
            INTEGER, INTENT(IN) :: JDATE, JTIME
            INTEGER, INTENT(IN) :: TSTEP(2)
         END SUBROUTINE ZADV_CAD
         SUBROUTINE XADV_CAD(LGRID, JDATE, JTIME, TSTEP, NADVSS)
            IMPLICIT NONE
            REAL, POINTER       :: LGRID(:,:,:,:)
            INTEGER, INTENT(IN) :: JDATE, JTIME
            INTEGER, INTENT(IN) :: TSTEP(2)
            INTEGER, INTENT(IN) :: NADVSS(:)
         END SUBROUTINE XADV_CAD
         SUBROUTINE YADV_CAD(LGRID, JDATE, JTIME, TSTEP, NADVSS)
            IMPLICIT NONE
            REAL, POINTER       :: LGRID(:,:,:,:)
            INTEGER, INTENT(IN) :: JDATE, JTIME
            INTEGER, INTENT(IN) :: TSTEP(2)
            INTEGER, INTENT(IN) :: NADVSS(:)
         END SUBROUTINE YADV_CAD
         SUBROUTINE VDIFF_ADJ(LGRID, EMGRID, JDATE, JTIME, TSTEP)
            IMPLICIT NONE
            REAL, POINTER       :: LGRID(:,:,:,:), EMGRID(:,:,:,:)
            INTEGER, INTENT(IN) :: JDATE, JTIME
            INTEGER, INTENT(IN) :: TSTEP(2)
         END SUBROUTINE VDIFF_ADJ
         SUBROUTINE ADJADV_ADJ(LGRID, JDATE, JTIME, TSTEP)
            IMPLICIT NONE
            REAL, POINTER       :: LGRID(:,:,:,:)
            INTEGER, INTENT(IN) :: JDATE, JTIME
            INTEGER, INTENT(IN) :: TSTEP(2)
         END SUBROUTINE ADJADV_ADJ
         SUBROUTINE COUPLE(LGRID, JDATE, JTIME, TSTEP)
            IMPLICIT NONE
            REAL, POINTER       :: LGRID(:,:,:,:)
            INTEGER, INTENT(IN) :: JDATE, JTIME
            INTEGER, INTENT(IN) :: TSTEP(2)
         END SUBROUTINE COUPLE
         SUBROUTINE DECOUPLE(LGRID, JDATE, JTIME, TSTEP)
            IMPLICIT NONE
            REAL, POINTER       :: LGRID(:,:,:,:)
            INTEGER, INTENT(IN) :: JDATE, JTIME
            INTEGER, INTENT(IN) :: TSTEP(2)
         END SUBROUTINE DECOUPLE
      END INTERFACE

C-----------------------------------------------------------------------

      IF ( FIRSTIME ) THEN
         FIRSTIME = .FALSE.
         LOGDEV = INIT3()
      END IF

      SDATE = JDATE
      STIME = JTIME

      CALL NEXTIME(SDATE, STIME, -TSTEP(2))
      WRITE(LOGDEV, '(/ 5X, A, I8, I7.6)') NMSG, SDATE, STIME

c$$$      CALL AERO(CGRID, SDATE, STIME, TSTEP)

      CALL CHEM_ADJ(LGRID, SDATE, STIME, TSTEP)

c$$$      CALL CLDPROC(CGRID, SDATE, STIME, TSTEP)

c$$$      CALL PING(SDATE, STIME, TSTEP)

c$$$      CALL DECOUPLE(LGRID, JDATE, JTIME, TSTEP)

      CALL HDIFF_AD(LGRID, SDATE, STIME, TSTEP)

c$$$      CALL ADJADV_ADJ(LGRID, SDATE, STIME, TSTEP)

      CALL ZADV_CAD(LGRID, SDATE, STIME, TSTEP)

      IF ( XFIRST ) THEN
         XFIRST = .FALSE.

         CALL XADV_CAD(LGRID, JDATE, JTIME, TSTEP, NADVSS)

         CALL YADV_CAD(LGRID, JDATE, JTIME, TSTEP, NADVSS)
      ELSE
         XFIRST = .TRUE.

         CALL YADV_CAD(LGRID, JDATE, JTIME, TSTEP, NADVSS)

         CALL XADV_CAD(LGRID, JDATE, JTIME, TSTEP, NADVSS)
      END IF

c$$$      CALL COUPLE(LGRID, SDATE, STIME, TSTEP)

      CALL VDIFF_ADJ(LGRID, EMGRID, SDATE, STIME, TSTEP)

      JDATE = SDATE
      JTIME = STIME

      RETURN
      END
