
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header: /home/aqm/ppercell/aq/cmaq_4.6/models/CCTM/src/driver/ctm/advstep.F,v 1.1.1.1 2006/09/22 18:19:30 sjr Exp $

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE SYNCSTEP(JDATE, JTIME, TSTEP, NREPS)

C-----------------------------------------------------------------------
C Function:
C    Use a fixed, user-input synchronization time step, TSTEP(2), and number
C    of synchronization time steps, NREPS, per output time step, TSTEP(1).

C Revision History:
C   01/10/09 by Peter Percell at UH-IMAQS:
C   - Adapted from advstep.F for use with adjoint/4dvar code
C-----------------------------------------------------------------------

      IMPLICIT NONE

C Includes:
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations

C Arguments:
      INTEGER, INTENT(IN) :: JDATE ! current model simulation date (YYYYDDD)
      INTEGER, INTENT(IN) :: JTIME ! current model simulation time (HHMMSS)
      INTEGER, INTENT(INOUT) :: TSTEP(2) ! time step vector (HHMMSS)
                                !   TSTEP(1) = output time step
                                !   TSTEP(2) = synchronization time step
      INTEGER, INTENT(OUT) :: NREPS ! number of sync steps per output step

C External Functions (not already declared by IODECL3.EXT):
      INTEGER, EXTERNAL :: SEC2TIME, TIME2SEC, ENVINT

C Local Variables:
      LOGICAL, SAVE :: FIRSTIME = .TRUE.
      INTEGER, SAVE :: LOGDEV   ! unit number for log device

      CHARACTER(16) :: PNAME = 'SYNCSTEP'
      CHARACTER(96) :: XMSG = ' '

      INTEGER, SAVE :: SYNC_STEP ! user-input synchronization time step (sec)

      INTEGER, SAVE :: OSTEP    ! output time step (sec)

      INTEGER :: STATUS         ! ENVINT status

      CHARACTER(80) :: VARDESC  ! environment variable description

C environment variable sync step
      CHARACTER(16) :: CTM_SYNC_STEP = 'CTM_SYNC_STEP'

C-----------------------------------------------------------------------

      IF ( FIRSTIME ) THEN
         FIRSTIME = .FALSE.
         LOGDEV = INIT3()

         SYNC_STEP = 600        ! default
         VARDESC = 'Synchronization Time Step (sec)'
         SYNC_STEP = ENVINT(CTM_SYNC_STEP, VARDESC, SYNC_STEP, STATUS)
         IF ( STATUS /= 0 ) WRITE(LOGDEV, '(5X, A)') VARDESC
         IF ( STATUS .EQ. 1 ) THEN
            XMSG = 'Environment variable improperly formatted'
            CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT2)
         END IF

C Output time step (sec)
         OSTEP = TIME2SEC(TSTEP(1))

C Make sure that the synchronization time step, SYNC_STEP, divides the output
C time step
         IF ( MOD(OSTEP, SYNC_STEP) /= 0 ) THEN
            WRITE(LOGDEV, '(/ 5X, A, I8, A, I8)')
     &           'User-input synchronization time step: ', SYNC_STEP,
     &           ' does not divide output time step: ', OSTEP
            XMSG = ' '
            CALL M3EXIT(PNAME, JDATE, JTIME, XMSG, XSTAT3)
         END IF
      END IF                    ! FIRSTIME

      WRITE(LOGDEV, '(/ 5X, A, I8, A, I6.6)')
     &     'From SYNCSTEP - date/time: ', JDATE, '/', JTIME

C Set the synchronization time step, TSTEP(2), and the corresponding number
C of sync steps per output step, NREPS
      TSTEP(2) = SEC2TIME(SYNC_STEP)
      NREPS = OSTEP / SYNC_STEP

      WRITE(LOGDEV, '(/ 5X, A, I6.6 / 5X, A, I3 /)')
     &     'User-input synchronization step (HHMMSS): ', TSTEP(2),
     &     'Number of synchronization steps per output step: ', NREPS

      END SUBROUTINE SYNCSTEP
